@@ -52,7 +52,7 @@ fn main() {
     }
 
     #[cfg(any(target_arch = "x86_64", target_arch = "aarch64"))]
-    if let Some(64) = python_config.pointer_width {
+    if matches!(python_config.pointer_width, Some(64)) {
         println!("cargo:rustc-cfg=feature=\"inline_int\"");
     }
 
@@ -75,7 +75,9 @@ fn main() {
             }
             Err(_) => {
                 if env::var("CARGO_FEATURE_YYJSON").is_ok() {
-                    panic!("yyjson was enabled but the build failed. To build with a different backend do not specify the feature.")
+                    panic!(
+                        "yyjson was enabled but the build failed. To build with a different backend do not specify the feature."
+                    )
                 }
             }
         }
