@@ -1,6 +1,8 @@
 # SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 
+import sys
+
 import pytest
 
 import orjson
@@ -12,7 +14,9 @@ except ImportError:
 
 
 def numpy_default(obj):
-    return obj.tolist()
+    if isinstance(obj, numpy.ndarray):
+        return obj.tolist()
+    raise TypeError
 
 
 @pytest.mark.skipif(numpy is None, reason="numpy is not installed")
@@ -114,6 +118,94 @@ class TestNumpy:
             == b"[1.0,3.4028235e38]"
         )
 
+    def test_numpy_array_d1_f16(self):
+        assert (
+            orjson.dumps(
+                numpy.array([-1.0, 0.0009765625, 1.0, 65504.0], numpy.float16),
+                option=orjson.OPT_SERIALIZE_NUMPY,
+            )
+            == b"[-1.0,0.0009765625,1.0,65504.0]"
+        )
+
+    def test_numpy_array_f16_roundtrip(self):
+        ref = [
+            -1.0,
+            -2.0,
+            0.000000059604645,
+            0.000060975552,
+            0.00006103515625,
+            0.0009765625,
+            0.33325195,
+            0.99951172,
+            1.0,
+            1.00097656,
+            65504.0,
+        ]
+        obj = numpy.array(ref, numpy.float16)  # type: ignore
+        serialized = orjson.dumps(
+            obj,
+            option=orjson.OPT_SERIALIZE_NUMPY,
+        )
+        deserialized = numpy.array(orjson.loads(serialized), numpy.float16)  # type: ignore
+        assert numpy.array_equal(obj, deserialized)
+
+    def test_numpy_array_f16_edge(self):
+        assert (
+            orjson.dumps(
+                numpy.array(
+                    [
+                        numpy.inf,
+                        -numpy.inf,
+                        numpy.nan,
+                        -0.0,
+                        0.0,
+                        numpy.pi,
+                    ],
+                    numpy.float16,
+                ),
+                option=orjson.OPT_SERIALIZE_NUMPY,
+            )
+            == b"[null,null,null,-0.0,0.0,3.140625]"
+        )
+
+    def test_numpy_array_f32_edge(self):
+        assert (
+            orjson.dumps(
+                numpy.array(
+                    [
+                        numpy.inf,
+                        -numpy.inf,
+                        numpy.nan,
+                        -0.0,
+                        0.0,
+                        numpy.pi,
+                    ],
+                    numpy.float32,
+                ),
+                option=orjson.OPT_SERIALIZE_NUMPY,
+            )
+            == b"[null,null,null,-0.0,0.0,3.1415927]"
+        )
+
+    def test_numpy_array_f64_edge(self):
+        assert (
+            orjson.dumps(
+                numpy.array(
+                    [
+                        numpy.inf,
+                        -numpy.inf,
+                        numpy.nan,
+                        -0.0,
+                        0.0,
+                        numpy.pi,
+                    ],
+                    numpy.float64,
+                ),
+                option=orjson.OPT_SERIALIZE_NUMPY,
+            )
+            == b"[null,null,null,-0.0,0.0,3.141592653589793]"
+        )
+
     def test_numpy_array_d1_f64(self):
         assert (
             orjson.dumps(
@@ -375,13 +467,10 @@ class TestNumpy:
             )
 
     def test_numpy_array_unsupported_dtype(self):
-        array = numpy.array([[1, 2], [3, 4]], numpy.float16)
+        array = numpy.array([[1, 2], [3, 4]], numpy.csingle)  # type: ignore
         with pytest.raises(orjson.JSONEncodeError) as cm:
             orjson.dumps(array, option=orjson.OPT_SERIALIZE_NUMPY)
         assert "unsupported datatype in numpy array" in str(cm)
-        assert orjson.dumps(
-            array, default=numpy_default, option=orjson.OPT_SERIALIZE_NUMPY
-        ) == orjson.dumps(array.tolist())
 
     def test_numpy_array_d1(self):
         array = numpy.array([1])
@@ -602,6 +691,12 @@ class TestNumpy:
             == b"18446744073709551615"
         )
 
+    def test_numpy_scalar_float16(self):
+        assert (
+            orjson.dumps(numpy.float16(1.0), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b"1.0"
+        )
+
     def test_numpy_scalar_float32(self):
         assert (
             orjson.dumps(numpy.float32(1.0), option=orjson.OPT_SERIALIZE_NUMPY)
@@ -623,82 +718,337 @@ class TestNumpy:
             == b'{"a":true,"b":false}'
         )
 
-    def test_numpy_datetime(self):
+    def test_numpy_datetime_year(self):
+        assert (
+            orjson.dumps(numpy.datetime64("2021"), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_month(self):
+        assert (
+            orjson.dumps(numpy.datetime64("2021-01"), option=orjson.OPT_SERIALIZE_NUMPY)
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_day(self):
         assert (
             orjson.dumps(
-                {
-                    "year": numpy.datetime64("2021"),
-                    "month": numpy.datetime64("2021-01"),
-                    "day": numpy.datetime64("2021-01-01"),
-                    "hour": numpy.datetime64("2021-01-01T00"),
-                    "minute": numpy.datetime64("2021-01-01T00:00"),
-                    "second": numpy.datetime64("2021-01-01T00:00:00"),
-                    "milli": numpy.datetime64("2021-01-01T00:00:00.172"),
-                    "micro": numpy.datetime64("2021-01-01T00:00:00.172576"),
-                    "nano": numpy.datetime64("2021-01-01T00:00:00.172576789"),
-                },
+                numpy.datetime64("2021-01-01"), option=orjson.OPT_SERIALIZE_NUMPY
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_hour(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00"), option=orjson.OPT_SERIALIZE_NUMPY
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_minute(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00"), option=orjson.OPT_SERIALIZE_NUMPY
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_second(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00"),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
-            == b'{"year":"2021-01-01T00:00:00","month":"2021-01-01T00:00:00","day":"2021-01-01T00:00:00","hour":"2021-01-01T00:00:00","minute":"2021-01-01T00:00:00","second":"2021-01-01T00:00:00","milli":"2021-01-01T00:00:00.172000","micro":"2021-01-01T00:00:00.172576","nano":"2021-01-01T00:00:00.172576"}'
+            == b'"2021-01-01T00:00:00"'
         )
 
-    def test_numpy_datetime_naive_utc(self):
+    def test_numpy_datetime_milli(self):
         assert (
             orjson.dumps(
-                {
-                    "year": numpy.datetime64("2021"),
-                    "month": numpy.datetime64("2021-01"),
-                    "day": numpy.datetime64("2021-01-01"),
-                    "hour": numpy.datetime64("2021-01-01T00"),
-                    "minute": numpy.datetime64("2021-01-01T00:00"),
-                    "second": numpy.datetime64("2021-01-01T00:00:00"),
-                    "milli": numpy.datetime64("2021-01-01T00:00:00.172"),
-                    "micro": numpy.datetime64("2021-01-01T00:00:00.172576"),
-                    "nano": numpy.datetime64("2021-01-01T00:00:00.172576789"),
-                },
+                numpy.datetime64("2021-01-01T00:00:00.172"),
+                option=orjson.OPT_SERIALIZE_NUMPY,
+            )
+            == b'"2021-01-01T00:00:00.172000"'
+        )
+
+    def test_numpy_datetime_micro(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172576"),
+                option=orjson.OPT_SERIALIZE_NUMPY,
+            )
+            == b'"2021-01-01T00:00:00.172576"'
+        )
+
+    def test_numpy_datetime_nano(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172576789"),
+                option=orjson.OPT_SERIALIZE_NUMPY,
+            )
+            == b'"2021-01-01T00:00:00.172576"'
+        )
+
+    def test_numpy_datetime_naive_utc_year(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021"),
                 option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
             )
-            == b'{"year":"2021-01-01T00:00:00+00:00","month":"2021-01-01T00:00:00+00:00","day":"2021-01-01T00:00:00+00:00","hour":"2021-01-01T00:00:00+00:00","minute":"2021-01-01T00:00:00+00:00","second":"2021-01-01T00:00:00+00:00","milli":"2021-01-01T00:00:00.172000+00:00","micro":"2021-01-01T00:00:00.172576+00:00","nano":"2021-01-01T00:00:00.172576+00:00"}'
+            == b'"2021-01-01T00:00:00+00:00"'
         )
 
-    def test_numpy_datetime_naive_utc_utc_z(self):
+    def test_numpy_datetime_naive_utc_month(self):
         assert (
             orjson.dumps(
-                {
-                    "year": numpy.datetime64("2021"),
-                    "month": numpy.datetime64("2021-01"),
-                    "day": numpy.datetime64("2021-01-01"),
-                    "hour": numpy.datetime64("2021-01-01T00"),
-                    "minute": numpy.datetime64("2021-01-01T00:00"),
-                    "second": numpy.datetime64("2021-01-01T00:00:00"),
-                    "milli": numpy.datetime64("2021-01-01T00:00:00.172"),
-                    "micro": numpy.datetime64("2021-01-01T00:00:00.172576"),
-                    "nano": numpy.datetime64("2021-01-01T00:00:00.172576789"),
-                },
+                numpy.datetime64("2021-01"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
+            )
+            == b'"2021-01-01T00:00:00+00:00"'
+        )
+
+    def test_numpy_datetime_naive_utc_day(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
+            )
+            == b'"2021-01-01T00:00:00+00:00"'
+        )
+
+    def test_numpy_datetime_naive_utc_hour(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
+            )
+            == b'"2021-01-01T00:00:00+00:00"'
+        )
+
+    def test_numpy_datetime_naive_utc_minute(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
+            )
+            == b'"2021-01-01T00:00:00+00:00"'
+        )
+
+    def test_numpy_datetime_naive_utc_second(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
+            )
+            == b'"2021-01-01T00:00:00+00:00"'
+        )
+
+    def test_numpy_datetime_naive_utc_milli(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
+            )
+            == b'"2021-01-01T00:00:00.172000+00:00"'
+        )
+
+    def test_numpy_datetime_naive_utc_micro(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172576"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
+            )
+            == b'"2021-01-01T00:00:00.172576+00:00"'
+        )
+
+    def test_numpy_datetime_naive_utc_nano(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172576789"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NAIVE_UTC,
+            )
+            == b'"2021-01-01T00:00:00.172576+00:00"'
+        )
+
+    def test_numpy_datetime_naive_utc_utc_z_year(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021"),
                 option=orjson.OPT_SERIALIZE_NUMPY
                 | orjson.OPT_NAIVE_UTC
                 | orjson.OPT_UTC_Z,
             )
-            == b'{"year":"2021-01-01T00:00:00Z","month":"2021-01-01T00:00:00Z","day":"2021-01-01T00:00:00Z","hour":"2021-01-01T00:00:00Z","minute":"2021-01-01T00:00:00Z","second":"2021-01-01T00:00:00Z","milli":"2021-01-01T00:00:00.172000Z","micro":"2021-01-01T00:00:00.172576Z","nano":"2021-01-01T00:00:00.172576Z"}'
+            == b'"2021-01-01T00:00:00Z"'
         )
 
-    def test_numpy_datetime_omit_microseconds(self):
+    def test_numpy_datetime_naive_utc_utc_z_month(self):
         assert (
             orjson.dumps(
-                {
-                    "year": numpy.datetime64("2021"),
-                    "month": numpy.datetime64("2021-01"),
-                    "day": numpy.datetime64("2021-01-01"),
-                    "hour": numpy.datetime64("2021-01-01T00"),
-                    "minute": numpy.datetime64("2021-01-01T00:00"),
-                    "second": numpy.datetime64("2021-01-01T00:00:00"),
-                    "milli": numpy.datetime64("2021-01-01T00:00:00.172"),
-                    "micro": numpy.datetime64("2021-01-01T00:00:00.172576"),
-                    "nano": numpy.datetime64("2021-01-01T00:00:00.172576789"),
-                },
+                numpy.datetime64("2021-01"),
+                option=orjson.OPT_SERIALIZE_NUMPY
+                | orjson.OPT_NAIVE_UTC
+                | orjson.OPT_UTC_Z,
+            )
+            == b'"2021-01-01T00:00:00Z"'
+        )
+
+    def test_numpy_datetime_naive_utc_utc_z_day(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01"),
+                option=orjson.OPT_SERIALIZE_NUMPY
+                | orjson.OPT_NAIVE_UTC
+                | orjson.OPT_UTC_Z,
+            )
+            == b'"2021-01-01T00:00:00Z"'
+        )
+
+    def test_numpy_datetime_naive_utc_utc_z_hour(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00"),
+                option=orjson.OPT_SERIALIZE_NUMPY
+                | orjson.OPT_NAIVE_UTC
+                | orjson.OPT_UTC_Z,
+            )
+            == b'"2021-01-01T00:00:00Z"'
+        )
+
+    def test_numpy_datetime_naive_utc_utc_z_minute(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00"),
+                option=orjson.OPT_SERIALIZE_NUMPY
+                | orjson.OPT_NAIVE_UTC
+                | orjson.OPT_UTC_Z,
+            )
+            == b'"2021-01-01T00:00:00Z"'
+        )
+
+    def test_numpy_datetime_naive_utc_utc_z_second(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00"),
+                option=orjson.OPT_SERIALIZE_NUMPY
+                | orjson.OPT_NAIVE_UTC
+                | orjson.OPT_UTC_Z,
+            )
+            == b'"2021-01-01T00:00:00Z"'
+        )
+
+    def test_numpy_datetime_naive_utc_utc_z_milli(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172"),
+                option=orjson.OPT_SERIALIZE_NUMPY
+                | orjson.OPT_NAIVE_UTC
+                | orjson.OPT_UTC_Z,
+            )
+            == b'"2021-01-01T00:00:00.172000Z"'
+        )
+
+    def test_numpy_datetime_naive_utc_utc_z_micro(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172576"),
+                option=orjson.OPT_SERIALIZE_NUMPY
+                | orjson.OPT_NAIVE_UTC
+                | orjson.OPT_UTC_Z,
+            )
+            == b'"2021-01-01T00:00:00.172576Z"'
+        )
+
+    def test_numpy_datetime_naive_utc_utc_z_nano(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172576789"),
+                option=orjson.OPT_SERIALIZE_NUMPY
+                | orjson.OPT_NAIVE_UTC
+                | orjson.OPT_UTC_Z,
+            )
+            == b'"2021-01-01T00:00:00.172576Z"'
+        )
+
+    def test_numpy_datetime_omit_microseconds_year(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021"),
                 option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
             )
-            == b'{"year":"2021-01-01T00:00:00","month":"2021-01-01T00:00:00","day":"2021-01-01T00:00:00","hour":"2021-01-01T00:00:00","minute":"2021-01-01T00:00:00","second":"2021-01-01T00:00:00","milli":"2021-01-01T00:00:00","micro":"2021-01-01T00:00:00","nano":"2021-01-01T00:00:00"}'
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_omit_microseconds_month(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_omit_microseconds_day(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_omit_microseconds_hour(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_omit_microseconds_minute(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_omit_microseconds_second(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_omit_microseconds_milli(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_omit_microseconds_micro(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172576"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
+            )
+            == b'"2021-01-01T00:00:00"'
+        )
+
+    def test_numpy_datetime_omit_microseconds_nano(self):
+        assert (
+            orjson.dumps(
+                numpy.datetime64("2021-01-01T00:00:00.172576789"),
+                option=orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_OMIT_MICROSECONDS,
+            )
+            == b'"2021-01-01T00:00:00"'
         )
 
     def test_numpy_datetime_nat(self):
@@ -708,7 +1058,7 @@ class TestNumpy:
             orjson.dumps([numpy.datetime64("NaT")], option=orjson.OPT_SERIALIZE_NUMPY)
 
     def test_numpy_repeated(self):
-        data = numpy.array([[[1, 2], [3, 4], [5, 6], [7, 8]]], numpy.int64)
+        data = numpy.array([[[1, 2], [3, 4], [5, 6], [7, 8]]], numpy.int64)  # type: ignore
         for _ in range(0, 3):
             assert (
                 orjson.dumps(
@@ -771,3 +1121,12 @@ class TestNumpyEquivalence:
                 [-1.7976931348623157e308, 1.7976931348623157e308], numpy.float64
             )
         )
+
+
+@pytest.mark.skipif(numpy is None, reason="numpy is not installed")
+class NumpyEndianness:
+    def test_numpy_array_dimension_zero(self):
+        wrong_endianness = ">" if sys.byteorder == "little" else "<"
+        array = numpy.array([0, 1, 0.4, 5.7], dtype=f"{wrong_endianness}f8")
+        with pytest.raises(orjson.JSONEncodeError):
+            orjson.dumps(array, option=orjson.OPT_SERIALIZE_NUMPY)
